program dipole
  implicit none

  integer ipar, ioper, imu, iterm, i, j, parmax(3), par_ind(3,9,235)
  
  double precision f(3), local(9), xyz(5,3)
  double precision par(3,235)
  character(30) buf30

  double precision y(9), xi(9,6), term_dip(3), vtot, r0e, r1e, betae, tau12, tau13, tau23, deg, pi
  double precision dip(3), mu(3), nu(3), tmat(4,3), amat(3,3), ainv(3,3)

  integer            :: k_ind(9), lwork, info, nsv
  double precision              :: dx(4,3)
  double precision       :: dmat(3,3), dipd(3,1), work(64*3), sv(3), svtol
  character(20)      :: txt

  par_ind=0
  par = 0.0d0

  ! read parameters

  read*, parmax(1) ! number of parameters mu_x
  read*, parmax(2) ! number of parameters mu_y
  read*, parmax(3) ! number of parameters mu_z

  do ipar=1, parmax(1)
    read*, buf30, par_ind(1,1:9,ipar), par(1,ipar)
  enddo
  do ipar=1, parmax(2)
    read*, buf30, par_ind(2,1:9,ipar), par(2,ipar)
  enddo
  do ipar=1, parmax(3)
    read*, buf30, par_ind(3,1:9,ipar), par(3,ipar)
  enddo

  ! Introduce a sample geometry in internal coordinates

  local(1) = 1.385000 !r0(C-F)
  local(2) = 1.086928 !r1(C-H1)
  local(3) = 1.086928 !r2(C-H2)
  local(4) = 1.086928 !r3(C-H3)
  local(5) = 108.773410 !beta1(H1-C-F)
  local(6) = 108.773410 !beta2(H2-C-F)
  local(7) = 108.773410 !beta3(H3-C-F)
  local(8) = 120.000000 !tau12(H1-C-F-H2)
  local(9) = 120.000000 !tau13(H1-C-F-H3)

  ! Same geometry in Cartesian coordinates
  !
  xyz = reshape( (/ &
  0.0000000000,      0.0000000000,      0.0000000000,&
  0.0000000000,      0.0000000000,      1.3850000000,&
  0.0000000000,      1.0291020350,     -0.3498020565,&
  0.8912285054,     -0.5145510175,     -0.3498020565,&
 -0.8912285054,     -0.5145510175,     -0.3498020565/),(/5,3/),order=(/2,1/))
  !
  ! which is equivalent to
  !
  !xyz(1,1) =  0.0000000000 ; xyz(1,2) =  0.0000000000 ; xyz(1,3) =  0.0000000000
  !xyz(2,1) =  0.0000000000 ; xyz(2,2) =  0.0000000000 ; xyz(2,3) =  1.3850000000
  !xyz(3,1) =  0.0000000000 ; xyz(3,2) =  1.0291020350 ; xyz(3,3) = -0.3498020565
  !xyz(4,1) =  0.8912285054 ; xyz(4,2) = -0.5145510175 ; xyz(4,3) = -0.3498020565
  !xyz(5,1) = -0.8912285054 ; xyz(5,2) = -0.5145510175 ; xyz(5,3) = -0.3498020565

  pi = 4.0d0 * datan2(1.0d0,1.0d0)

  deg = pi/180.0d0

  ! 'equilibrium' parameters

  r0e   = par(1,1)
  r1e   = par(1,2)
  betae = par(1,3)

  ! DMS coordinates

  y(1) = (local(1)-r0e)
  y(2) = (local(2)-r1e)
  y(3) = (local(3)-r1e)
  y(4) = (local(4)-r1e)
  y(5) = (local(5)-betae)*deg
  y(6) = (local(6)-betae)*deg
  y(7) = (local(7)-betae)*deg

  tau12 = local(8)*deg
  tau13 = local(9)*deg
  tau23 = 2.0d0*pi-tau12-tau13

  y(8) = (2.0d0*tau23-tau13-tau12)/sqrt(6.0d0)
  y(9) = (tau13-tau12)/sqrt(2.0d0)

  do ioper=1, 6
    call ch3f_symmetry_transformation_local(ioper,y,xi(:,ioper))
  enddo

  ! compute symmetry-adapted dipole moment functions

  dip(1:3) = 0.0d0

  do imu=1, 3
    mu(1:3) = 0.0d0
    do iterm=4, parmax(imu)
      k_ind(1:9) = par_ind(imu,1:9,iterm)
      term_dip = 0.0d0
      do ioper=1, 6
        mu(imu) = product(xi(1:9,ioper)**k_ind(1:9))
        call ch3f_symmetry_transformation_dipole(ioper,mu,nu)
        term_dip(:) = term_dip(:) + nu(:)
      enddo
      dip(:) = dip(:) + term_dip(:) * par(imu,iterm)
    enddo ! iterm
  enddo ! imu

  ! form transformation from Cartesian directions to symmetry-adapted
  ! bond-projections
  ! AMAT * DIP_{X,Y,Z} = DIP_{S1,S2,S3}

  tmat(1,1:3) = (xyz(2,1:3)-xyz(1,1:3))/local(1) ! C--F
  tmat(2,1:3) = (xyz(3,1:3)-xyz(1,1:3))/local(2) ! C--H1
  tmat(3,1:3) = (xyz(4,1:3)-xyz(1,1:3))/local(3) ! C--H2
  tmat(4,1:3) = (xyz(5,1:3)-xyz(1,1:3))/local(4) ! C--H3

  amat(1,:) = (tmat(2,:)*2.0d0-tmat(3,:)-tmat(4,:))/sqrt(6.0d0)
  amat(2,:) = (tmat(3,:)-tmat(4,:))/sqrt(2.0d0)
  amat(3,:) =  tmat(1,:)

  ! find DIP_{X,Y,Z} = AMAT^{-1} * DIP_{S1,S2,S3}

  call MLlinurark(3,amat,dip(:),f,info)

  write(*,'(9(1x,f12.6),3(1x,f16.8))') local(1:9), f(1:3)

end program dipole


  subroutine ch3f_symmetry_transformation_local(ioper,src,dst)
    implicit none
    !
    integer,intent(in)    :: ioper  ! group operation  
    double precision,intent(in)      :: src(1:9)
    double precision,intent(out)     :: dst(1:9)
    !
    double precision         :: repres(12,2,2),a,b,e,o
    !
    dst(1) = src(1)
    !
    select case(ioper)
    !
    case (1) ! identity 
 
      dst = src
 
    case (3) ! (123)
 
      dst(2) = src(3)
      dst(3) = src(4)
      dst(4) = src(2)
      !
      dst(5) = src(6)
      dst(6) = src(7)
      dst(7) = src(5)
 
    case (2) ! (321)
 
      dst(2) = src(4)
      dst(3) = src(2)
      dst(4) = src(3)
      !
      dst(5) = src(7)
      dst(6) = src(5)
      dst(7) = src(6)
 
    case (6) ! (12)
 
      dst(2) = src(3)
      dst(3) = src(2)
      dst(4) = src(4)
      !
      dst(5) = src(6)
      dst(6) = src(5)
      dst(7) = src(7)
 
    case (5) ! (13)
 
      dst(2) = src(4)
      dst(3) = src(3)
      dst(4) = src(2)
      !
      dst(5) = src(7)
      dst(6) = src(6)
      dst(7) = src(5)
 
    case (4) ! (23)
 
      dst(2) = src(2)
      dst(3) = src(4)
      dst(4) = src(3)
      !
      dst(5) = src(5)
      dst(6) = src(7)
      dst(7) = src(6)
 
    case default
 
    end select
    !
    a = 0.5d0 ; b = 0.5d0*sqrt(3.0d0) ; e = 1.0d0 ; o = 0.0d0
    !
    repres ( 1,:,:)= reshape((/ e, o,  &
                                o, e/),(/2,2/))
    !
    repres ( 3,:,:)= reshape((/-a,-b,  &
                                b,-a/),(/2,2/))
    !
    repres ( 2,:,:)= reshape((/-a, b,  &
                               -b,-a/),(/2,2/))
    !
    repres ( 4,:,:)= reshape((/ e, o,  &
                                o,-e/),(/2,2/))
    !
    repres ( 6,:,:)= reshape((/-a, b,  &
                                b, a/),(/2,2/))
    !
    repres ( 5,:,:)= reshape((/-a,-b,  &
                               -b, a/),(/2,2/))
    !
    dst(8) = repres(ioper,1,1)*src(8)+repres(ioper,1,2)*src(9)
    dst(9) = repres(ioper,2,1)*src(8)+repres(ioper,2,2)*src(9)

  end subroutine ch3f_symmetry_transformation_local


  subroutine ch3f_symmetry_transformation_dipole(ioper,src,dst)

    implicit none
    !
    integer, intent(in) :: ioper  ! group operation
    double precision, intent(in)  :: src(1:3)
    double precision, intent(out) :: dst(1:3)
    !
    double precision :: repres(6,3,3),a,b,e,o
    !
    a = 0.5d0
    b = 0.5d0*sqrt(3.0d0)
    e = 1.0d0
    o = 0.0d0
    !
    repres = 0
    !
    repres(:,3,3) = e !no operations on mu_z
    !
    repres(1,1,1) = e !identity
    repres(1,2,2) = e
    !
    repres(2,1,1) = -a !(321)
    repres(2,1,2) = -b
    repres(2,2,1) =  b
    repres(2,2,2) = -a
    !
    repres(3,1,1) = -a !(123)
    repres(3,1,2) =  b
    repres(3,2,1) = -b
    repres(3,2,2) = -a
    !
    repres(4,1,1) =  e !(23)
    repres(4,1,2) =  o
    repres(4,2,1) =  o
    repres(4,2,2) = -e
    !
    repres(5,1,1) = -a !(13)
    repres(5,1,2) = -b
    repres(5,2,1) = -b
    repres(5,2,2) =  a
    !
    repres(6,1,1) = -a !(12)
    repres(6,1,2) =  b
    repres(6,2,1) =  b
    repres(6,2,2) =  a
    !
    dst = matmul(transpose(repres(ioper,:,:)),src)
    !
  end subroutine ch3f_symmetry_transformation_dipole

  subroutine MLlinurark(dimen,coeff,constant,solution,error)

    integer, parameter :: ik          = selected_int_kind(8)       ! "Normal" integers. This must map on
    integer, parameter :: rk          = selected_real_kind(12,25)  ! "Normal" reals and complex (complexi? :-)
    integer, parameter :: ark         = selected_real_kind(25,32)  ! "Accurate" reals and complex (complexi? :-)
 
    integer(ik),intent(in)  :: dimen
    real(ark),intent(in)  :: coeff(dimen,dimen),constant(dimen)
    real(ark),intent(out) :: solution(dimen)
    integer(ik),intent(out) :: error
    !
    real(ark)          :: a0(dimen,dimen)
    real(ark)          :: c
    integer(ik)        :: i1,i2,i,k8,k,l,k9
 
    !----- begin ----!
 
      if (verbose>=6) write(out,"('MLlinurark...')")
      !
      error=0
      !
      if (dimen==1) then
       !
       if (abs(coeff(1,1))<small_) then
          !write(out,"(i,'-th parameter is wrong, small a0(i,i) =  ',g18.8)")
          !i,coeff(i,i)
          error=i
          return
       endif
       !
       solution(1) = constant(1)/coeff(1,1)
       !
       return
       !
      endif
      !
      a0 = coeff
      solution = constant
      !
      do i=1,dimen
        c=0
        k8=i-1
        do k=1,k8
          c=c+a0(k,i)*a0(k,i)
        enddo
 
        if (c.ge.a0(i,i)) then
         !write(out,"(i,'-th parameter is wrong, c >= a0(i,i) =  ',g18.8)")
         !i,a0(i,i)
         error=i
         return
        endif
        !
        a0(i,i)=sqrt(a0(i,i)-c)
        if (abs(a0(i,i))<small_) then
           !write(out,"(i,'-th parameter is wrong, small a0(i,i) =  ',g18.8)")
           !i,a0(i,i)
           error=i
           return
        endif
        k8=i+1
        do l=k8,dimen
           k9=i-1
           c=0
           do k=1,k9
              c=c+a0(k,i)*a0(k,l)
           enddo
           a0(i,l)=(a0(l,i)-c)/a0(i,i)
        enddo
      enddo
      do i=1,dimen
        k8=i-1
        c=0
        do k=1,k8
           c=c+a0(k,i)*solution(k)
        enddo
        solution(i)=(solution(i)-c)/a0(i,i)
      enddo
      do i1=1,dimen
        i=1+dimen-i1
        k8=i+1
        c=0
        do k=k8,dimen
            c=c+a0(i,k)*solution(k)
        enddo
        solution(i)=(solution(i)-c)/a0(i,i)
      enddo
      !
      if (verbose>=6) write(out,"('...MLlinurark done!')")
      !
    end subroutine MLlinurark
