program poten
  implicit none

  integer ipar, parmax, info
  integer, allocatable :: par_ind(:,:)
  double precision f, local(9)
  double precision, allocatable :: par(:)
  character(30) buf30

  interface
  function poten_ch3f_hl(local,parmax,par,par_ind)
    integer parmax, par_ind(9,parmax)
    double precision local(9), par(parmax), poten_ch3f_hl
  end function poten_ch3f_hl
  end interface


  !read parameters

  read*, parmax
  allocate(par(parmax), par_ind(9,parmax), stat=info)
  if (info/=0) stop 'par, par_ind allocation error'

  do ipar=1, parmax
    read*, buf30, par_ind(1:9,ipar), par(ipar)
  enddo

  !read grid, print potential energy

  do
    read(*,*,end=4,err=4) local(1:9)
    f = poten_ch3f_hl(local, parmax, par, par_ind)
    write(*,'(9(1x,f12.6),2x,f14.6)') local(1:9), f
    cycle
4  exit
  enddo


end program poten

function poten_ch3f_hl(local,parmax,par,par_ind) result(f)
  implicit none

  integer parmax, par_ind(9,parmax)
  double precision local(9), par(parmax)
  double precision f

  integer ipar, ioper
  double precision y(9), xi(9), term(6), vtot, r0e, r1e, betae, a0, b0, tau12, tau13, tau23, deg, pi
!  double precision, parameter :: pi = 3.14159265359

  pi = 4.0d0 * datan2(1.0d0,1.0d0)

  deg = pi/180.0d0

  r0e   = par(1)
  r1e   = par(2)
  betae = par(3)
  a0    = par(4)
  b0    = par(5)

  y(1) = 1.0d0 - exp(-a0*(local(1)-r0e))
  y(2) = 1.0d0 - exp(-b0*(local(2)-r1e))
  y(3) = 1.0d0 - exp(-b0*(local(3)-r1e))
  y(4) = 1.0d0 - exp(-b0*(local(4)-r1e))
  y(5) = (local(5)-betae)*deg
  y(6) = (local(6)-betae)*deg
  y(7) = (local(7)-betae)*deg

  tau12 = local(8)*deg
  tau13 = local(9)*deg
  tau23 = 2.0d0*pi-tau12-tau13

  y(8) = (2.0d0*tau23-tau13-tau12)/sqrt(6.0d0)
  y(9) = (tau13-tau12)/sqrt(2.0d0)

  vtot = 0.0d0

  do ipar = 6, parmax

      do ioper =1,6
       !
       call ML_symmetry_transformation_XY3(ioper,y,xi)
       !
       term(ioper) = product(xi(1:9)**par_ind(1:9,ipar))
       !
      enddo   

      vtot = vtot + par(ipar)*(term(1) + term(2) + term(3) + term(4) + term(5) + term(6))

  enddo

  f = vtot

end function poten_ch3f_hl

  subroutine ML_symmetry_transformation_XY3(ioper,src,dst)
    implicit none
    !
    integer,intent(in)    :: ioper  ! group operation  
    double precision,intent(in)      :: src(1:9)
    double precision,intent(out)     :: dst(1:9)
    !
    double precision         :: repres(12,2,2),a,b,e,o
    !
    dst(1) = src(1)
    !
    select case(ioper)
    !
    case (1) ! identity 

      dst = src

    case (3) ! (123)

      dst(2) = src(3)
      dst(3) = src(4)
      dst(4) = src(2)
      !
      dst(5) = src(6)
      dst(6) = src(7)
      dst(7) = src(5)

    case (2) ! (321)

      dst(2) = src(4)
      dst(3) = src(2)
      dst(4) = src(3)
      !
      dst(5) = src(7)
      dst(6) = src(5)
      dst(7) = src(6)

    case (6) ! (12)

      dst(2) = src(3)
      dst(3) = src(2)
      dst(4) = src(4)
      !
      dst(5) = src(6)
      dst(6) = src(5)
      dst(7) = src(7)

    case (5) ! (13)

      dst(2) = src(4)
      dst(3) = src(3)
      dst(4) = src(2)
      !
      dst(5) = src(7)
      dst(6) = src(6)
      dst(7) = src(5)

    case (4) ! (23)

      dst(2) = src(2)
      dst(3) = src(4)
      dst(4) = src(3)
      !
      dst(5) = src(5)
      dst(6) = src(7)
      dst(7) = src(6)

    case default

    end select
    !
    a = 0.5d0 ; b = 0.5d0*sqrt(3.0d0) ; e = 1.0d0 ; o = 0.0d0
    !
    repres ( 1,:,:)= reshape((/ e, o,  &
                                o, e/),(/2,2/))
    !
    repres ( 3,:,:)= reshape((/-a,-b,  &
                                b,-a/),(/2,2/))
    !
    repres ( 2,:,:)= reshape((/-a, b,  &
                               -b,-a/),(/2,2/))
    !
    repres ( 4,:,:)= reshape((/ e, o,  &
                                o,-e/),(/2,2/))
    !
    repres ( 6,:,:)= reshape((/-a, b,  &
                                b, a/),(/2,2/))
    !
    repres ( 5,:,:)= reshape((/-a,-b,  &
                               -b, a/),(/2,2/))
    !
    dst(8) = repres(ioper,1,1)*src(8)+repres(ioper,1,2)*src(9)
    dst(9) = repres(ioper,2,1)*src(8)+repres(ioper,2,2)*src(9)

  end subroutine ML_symmetry_transformation_XY3
